@@ -125,16 +125,23 @@ insert_key_mapping_with_setter (GrlKeyID                       grl_key,
 void
 grl_tracker_setup_key_mappings (void)
 {
-  grl_metadata_key_tracker_urn =
-    grl_plugin_registry_register_metadata_key (grl_plugin_registry_get_default (),
-                                               g_param_spec_string ("tracker-urn",
-                                                                    "Tracker URN",
-                                                                    "Universal resource number in Tracker's store",
-                                                                    NULL,
-                                                                    G_PARAM_STATIC_STRINGS |
-                                                                    G_PARAM_READWRITE),
-                                               NULL);
+  GrlPluginRegistry *registry = grl_plugin_registry_get_default ();
 
+  /* Check if "tracker-urn" is registered; if not, then register it */
+  grl_metadata_key_tracker_urn =
+    grl_plugin_registry_lookup_metadata_key (registry, "tracker-urn");
+
+  if (grl_metadata_key_tracker_urn == NULL) {
+    grl_metadata_key_tracker_urn =
+      grl_plugin_registry_register_metadata_key (grl_plugin_registry_get_default (),
+                                                 g_param_spec_string ("tracker-urn",
+                                                                      "Tracker URN",
+                                                                      "Universal resource number in Tracker's store",
+                                                                      NULL,
+                                                                      G_PARAM_STATIC_STRINGS |
+                                                                      G_PARAM_READWRITE),
+                                                 NULL);
+  }
 
   grl_to_sparql_mapping = g_hash_table_new (g_direct_hash, g_direct_equal);
   sparql_to_grl_mapping = g_hash_table_new (g_str_hash, g_str_equal);
@@ -279,12 +286,10 @@ grl_tracker_setup_key_mappings (void)
                       "nie:contentAccessed(?urn)",
                       "media");
 
-#ifdef TRACKER_0_10_5
   insert_key_mapping (GRL_METADATA_KEY_LAST_POSITION,
                       "nfo:lastPlayedPosition",
                       "nfo:lastPlayedPosition(?urn)",
                       "media");
-#endif
 
   if (grl_tracker_upnp_present) {
     insert_key_mapping (GRL_METADATA_KEY_THUMBNAIL,
@@ -531,14 +536,15 @@ grl_tracker_build_grilo_media (const gchar *rdf_type)
       media = grl_media_box_new ();
     } else if (g_str_has_suffix (rdf_single_type[i], RDF_TYPE_FOLDER)) {
       media = grl_media_box_new ();
-    } else {
-      media = grl_media_new ();
     }
     i--;
   }
 
   g_strfreev (rdf_single_type);
 
+  if (!media)
+    media = grl_media_new ();
+
   return media;
 }
 
